clear;
// *****************************************************************************
function y=Porte(t)
    ind=find(t<-1/2 | t>1/2);
    y=ones(size(t,1),size(t,2));
    if length(ind)>0
        y(ind)=0;
    end
endfunction
// *****************************************************************************
function n=CANbipolaire(u,R,Umax)
    n=max(min(floor(2^R*u/(2*Umax)),2^(R-1)-1),-2^(R-1));
endfunction
// *****************************************************************************
function n=CNAbipolaire(u,R,Umax)
    n=u/2^R*2*Umax;
endfunction
// *****************************************************************************
function p=Arg(z)
    p=atan(imag(z),real(z));
endfunction
// *****************************************************************************
function y=SignalContinu(t)
    y=Porte((t-D/2)/D).*exp(-a*t).*sin(2*%pi*f0*t);
endfunction

// *****************************************************************************
// Simulation de la numérisation
// *****************************************************************************

// Paramètres de la numérisation
tDeb=0;tFin=10;  
Fe=250;    // Nombre de mesures par secondes
Nb=16;     // Convertisseur 16 bits
uMax=+5;   // Domaine de fonctionnement du convertisseur entre -uMax et +uMax V
// le signal continu est un sinus à la fréquence de 11Hz
f0=11;a=3;
te=0:1/(Fe):tFin; // Simulation de l'échantillonnage temporel
D=max(te);  // Durée de l'échantillonnage
N=length(te); // Nombre d'échantillons
ye=SignalContinu(te);
yn=CANbipolaire(ye,Nb,uMax);
yna=CNAbipolaire(yn,Nb,uMax);
figure(1);clf();
plot(te,yna,'b')
aa=gcf();
courbeYna=aa.children(1).children(1).children;
courbeYna.line_mode='off';
courbeYna.polyline_style=3;
courbeYna.mark_style=3;
legend('Signal numérique analogique')
xlabel('Temps (s)');
ylabel('Tension (V)');
title('Du continu au numérique')
aa=gca();
aa.data_bounds=[min(te) 1.1*min(yna); max(te) 1.1*max(yna)];
xgrid();
Yna=fft(yna,-1);
figure(2);clf();
subplot(2,1,1)
plot((-fix(N/2):N-fix(N/2)-1)/N,abs(fftshift(Yna)),'.')
xlabel('Fréquence normalisée')
ylabel('|TFD| (V)')
xgrid();
subplot(2,1,2)
plot((-fix(N/2):N-fix(N/2)-1)/N,Arg(fftshift(Yna)),'.')
xgrid();
// Calcul de la transformée de Fourier à partir de l'expression théorique
nu=(-fix(N/2):N-fix(N/2)-1)/N*Fe;
Xnu=(2*%pi*f0) ./ ((a+%i*2*%pi.*nu).^2+(2*%pi*f0).^2);
figure(3);clf();
subplot(2,1,1)
title('$T.F.D.\;et\;TF\; de\; sin(2\pi \nu_0 t)e^{-at}$')
plot((-fix(N/2):N-fix(N/2)-1)/N*Fe,abs(fftshift(Yna))/Fe,'b.')
plot(nu,abs(Xnu),'r')
xlabel('Fréquence (Hz)')
ylabel('$|X(\nu)|(Vs)$')
legend('$X(\frac{kF_e}N)$','$\frac{2\pi f_0}{(a+i2\pi\nu)^2+(2\pi f_0)^2}$')
xgrid();
subplot(2,1,2)
fn=(-fix(N/2):N-fix(N/2)-1)/N*Fe;
plot(fn,Arg(fftshift(Yna)),'b.')
plot(nu,Arg(Xnu),'r')
xlabel('Fréquence (Hz)')
ylabel('Phase (rd)')
xgrid();


