// ***************************************************************
//  Définition des fonctions
// ***************************************************************
function p=Arg(z)
    p=atan(imag(z),real(z));
endfunction
// ***************************************************************
function c=SFourier(f,T,n)
    c=[];
    t=[-T/2:T/20000:T/2];
    for i=1:length(n)
        w=1/T*eval(f).*exp(-%i*2*%pi*n(i)*t./T);
        c=[c inttrap(t,w)];
    end
endfunction
// ****************************************************************
function F=TFourier(f,t,nu)
    F=zeros(size(nu,1),size(nu,2));
    for i=1:length(nu)
        w=eval(f).*exp(-%i*2*%pi*nu(i)*t);
        z=inttrap(t,w);
        F(i)=z;
    end
endfunction
// ****************************************************************
function y=Porte(t)
    ind=find(t<-1/2 | t>1/2);
    y=ones(size(t,1),size(t,2));
    if length(ind)>0
        y(ind)=0;
    end
endfunction
// ****************************************************************
function y=SignalContinu(t)
    y=Porte((t-0.05)/(0.1));
endfunction


// ****************************************************************
//  Programme comparant Transformée de Fourier Discrète, 
//  Transformée de Fourier et série de Fourier
// ****************************************************************

// Paramètres de la numérisation
tDeb=0;tFin=1.5;  
D=tFin-tDeb;
Fe=120;    
te=0:1/(Fe):tFin; 
N=length(te);
ye=SignalContinu(te);
Ye=fft(ye);

f='SignalContinu(t)';
nu=-3/4*Fe:Fe/(3*N):3/4*Fe;
t=-tFin/2:0.001:1.5*tFin;
F=TFourier(f,t,nu);
T=D;
n=fix(T*min(nu)):fix(T*max(nu))
ck=SFourier(f,T,n);

figure(1);clf();
param3d(ones(size(t,1),size(t,2)),t,SignalContinu(t))
param3d(2*ones(size(t,1),size(t,2)),t,SignalContinu(pmodulo(t,T)))
param3d(3*ones(size(te,1),size(te,2)),te,ye,0,90)
aa=gca();
courbeYna=aa.children(1);
courbeYna.line_mode='on';
courbeYna.mark_style=3;
courbeYna.foreground=3;
courbeYna=aa.children(2);
courbeYna.thickness=3;
courbeYna.foreground=2;
courbeYna=aa.children(3);
courbeYna.thickness=3;
courbeYna.foreground=5;
xgrid();
legend('Signal continu transitoire','Signal périodique','Signal transitoire échantillonné')
title('Signaux : tansitoire, périodique, numérique') 
ylabel('Temps(s)');
zlabel('Signal (u;a.)');

figure(2);clf();
title('Fourier : série, transformée, transformée discrète') 
param3d(0.1*ones(size(nu,1),size(nu,2)),nu/Fe,abs(F))
param3d(0.20*ones(size(n,1),size(n,2)),n/T/Fe,T*abs(ck));
fn=(-fix(N/2):N-fix(N/2)-1)/N;;
param3d(0.30*ones(size(fn,1),size(fn,2)),fn,abs(fftshift(Ye))/Fe,0,90)
aa=gca();
courbeYna=aa.children(1);
courbeYna.line_mode='on';
courbeYna.mark_style=3;
courbeYna.foreground=3;
courbeYna=aa.children(2);
courbeYna.thickness=3;
courbeYna.foreground=2;
courbeYna=aa.children(3);
courbeYna.thickness=3;
courbeYna.foreground=5;
ylabel('Fréquence/Fe')
zlabel('$ |X(\nu)|$');
legend('TF Signal continu transitoire','SF Signal périodique','fft Signal transitoire échantillonné')
xgrid();
 
figure(3);clf();
plot(nu,Arg(F),'r')
plot(n/T,Arg(ck),'bo');
fn=(-fix(N/2):N-fix(N/2)-1)/N*Fe;;
plot(fn,Arg(fftshift(Ye)),'g+')
xlabel('Fréquence (Hz)')
ylabel('$ Arg(X(\nu))(rd)$');
xgrid();

k=0:N-1;n0=sum(ye)-1;
tfd=exp(-%i*%pi*k*n0/N).*sinc(%pi*k/N*(n0+1)) ./ sinc(%pi*k/N)*(n0+1);
disp('TFD analytique et fft');
disp(Ye(2),tfd(2));
disp('Série de Fourier analytique et SF estimée ');
T0=D/10;cn=exp(-%i*%pi*n*D/10/T).*sinc(%pi*n*T0/T)*T0/T
disp(cn(25),ck(25))
disp('Transformée de Fourier analytique et TSF estimée ');
Fnu=exp(-%i*%pi*nu*T0).*sinc(%pi*nu*T0)*T0;
disp(Fnu(57),F(57))
