clear;
// ************************************************************************
//  Définition des fonctions
// ************************************************************************
function y=Porte(t)
    ind=find(t<-1/2 | t>1/2); // Recherche des valeurs à l'extérieur de[-1/2,1/2]
    y=ones(t); // y matrice contenant des 1 de la même taille que t
    if length(ind)>0 // Les valeurs à l'extérieur de[-1/2,1/2] sont mises à 0
        y(ind)=0;
    end
endfunction

function n=CANbipolaire(u,R,Umax)
    n=max(min(floor(2^R*u/(2*Umax)),2^(R-1)-1),-2^(R-1));
endfunction

function n=CNAbipolaire(u,R,Umax)
    n=u/2^R*2*Umax;
endfunction

function y=SignalContinu(t)
    y=sin(2*%pi*f0*t);
endfunction

// *****************************************************************************
// Simulation de la numérisation
// *****************************************************************************

// Paramètres de la numérisation
tDeb=2;     // Instant de début de la numérisation en seconde
tFin=2.05;  // Instant de Fin de la numérisation en seconde
Fe=1000;    // Nombre de mesures par secondes
Nb=6;       // Convertisseur 6 bits
uMax=+5; // Domaine de fonctionnement du convertisseur entre -uMax et +uMax V
// le signal continu est un sinus à la fréquence de 110Hz
f0=110;
t=tDeb-(tFin-tDeb)/10:1/(Fe*100):tFin+(tFin-tDeb)/10; // simulation du temps continu
te=tDeb-(tFin-tDeb)/10:1/(Fe):tFin+(tFin-tDeb)/10; // Simulation de l'échantillonnage temporel
y=SignalContinu(t);  
ye=SignalContinu(te);

yf=Porte((te-(tDeb+tFin)/2)/(tFin-tDeb)).*ye;
ind=find(te>=tDeb & te<=tFin);
tn=te(ind);
yn=CANbipolaire(yf(ind),Nb,uMax);
yna=CNAbipolaire(yn,Nb,uMax);
// Graphique du signal analogique et discret dans la fenêtre n°1
figure(1);clf();
// le graphique est divisé en deux lignes et une colonne et on trace dans la première ligne
subplot(2,1,1);
// Le signal continu en rouge 
plot(t,y,'r')
// Le signal discret en bleu
plot(te,ye,'b')
a=gcf();
// Le mode de tracé du signal tracé est modifié afin d'obtenir des barres verticales
courbeYe=a.children(1).children(1).children;
// Les points ne sont plus reliés entre eux
courbeYe.line_mode='off';
// Voir la documentation Aide Scilab >> Graphiques > polygon > polyline properties
courbeYe.polyline_style=3;
courbeYe.mark_style=3;
legend('Signal continu','Signal discret')
xlabel('Temps (s)');
ylabel('Tension (V)');
title('Du continu au numérique')
a=gca();
// La zone visible du graphique est étendue
a.data_bounds=[min(t) 1.1*min(y); max(t) 1.1*max(y)];
// Ajout d'une grille
xgrid();
// On trace maintenant dans la deuxième ligne
subplot(2,1,2)
// Tracé des valeurs du signal numérique (valeurs de temps entre tDeb et tFin)
// en vert avec un trait d'épaisseur 2
plot(te(ind),yn,'g','thickness',2)
xlabel('Temps (s)');
ylabel('Sortie du convertisseur');
a=gca();
a.data_bounds=[min(t) 1.1*min(yn); max(t) 1.1*max(yn)];
a=gcf();
courbeYn=a.children(1).children(1).children;
courbeYn.line_mode='off';
courbeYn.polyline_style=3;
courbeYn.mark_style=3;
legend('Signal numérique')
xgrid();
// Graphique du signal analogique et du signal numérique dans la fenêtre n°2
figure(2);clf();
plot(t,y,'r')
plot(tn,yna,'b')
a=gcf();
courbeYna=a.children(1).children(1).children;
courbeYna.line_mode='off';
courbeYna.polyline_style=3;
courbeYna.mark_style=3;
legend('Signal continu','Signal Numérique analogique')
xlabel('Temps (s)');
ylabel('Tension (Vsignal discret)');
title('Du continu au numérique')
a=gca();
a.data_bounds=[min(t) 1.1*min(y); max(t) 1.1*max(y)];
xgrid();


