clear;


// ***************************************************************
//  Définition des fonctions
// ***************************************************************

function y=SignalContinu(t)
    y=sin(2*%pi*nu0*t)+1.4*sin(2*%pi*nu0*0.37*t);
endfunction

function y=Porte(t)
    ind=find(t<-1/2 | t>1/2);
    y=ones(size(t,1),size(t,2));
    if length(ind)>0
        y(ind)=0;
    end
endfunction

function y=Reconstruction(tpsContinu,taille,Fe,x)
    N=length(x);
        
    k=0:(N-1);
    y=[];
    for i=1:length(tpsContinu)
        k2=round(tpsContinu(i)*Fe);
        k1=max(k2-taille,0):min(k2+taille,N-1);
        y=[y sum(x(k1+1).*sinc(%pi*Fe*(tpsContinu(i)-k1/Fe)))];
    end
endfunction


// *****************************************************************************
// Programme calculant la valeur du signal analogique à partir des échantilllons
// *****************************************************************************
 
// Paramètres de la numérisation
tDeb=0;tFin=5;  
Fe=40;    
// Le signal continu est un sinus à la fréquence de 14Hz
nu0=14;a=0;
Te=1/Fe;
// Simulation de l'échantillonnage temporel
te=0:1/(Fe):tFin; 
// Valeur de temps pour tracer le signal analogique
t=tDeb-(tFin-tDeb)*0.1:1/(80*Fe):tFin+(tFin-tDeb)*0.1;
// Tirage aléatoire des valeurs de temps où sera estimé le signal analogique
tr=rand(0:1/(20*Fe):tFin)*(tFin-tDeb)+tDeb;
// Numérisation du signal
ye=SignalContinu(te);
// Signal continu
y=SignalContinu(t);
// Reconstruction du signal aux instants tr à partir des valeurs du signal numérique ye 
// échantillonné à Fe en utilisant 101=2*50+1 termes
yr=Reconstruction(tr,50,Fe,ye);
// Graphique avec le signal continu en rouge , le signal numérique en bleu 
// et le signal reconstruit en vert
figure(0);clf();
plot(te,ye,'b','thickness',2)
plot(t,y,'r','thickness',3);
plot(tr,yr,'g.')
legend('Signal numérique','Signal analogique','Valeurs estimées du signal analogique');
xlabel('Temps (s)')
ylabel('tension (V)');
title('Comparaison du signal continu avec signal reconstruit');
a=gcf();
courbeYe=a.children(1).children(4).children;
courbeYe.line_mode='off';
courbeYe.polyline_style=3;
courbeYe.mark_style=3;
xgrid();
// Graphique de l'errreur égale à la différence entre signal reconstruit et signal continu
figure(1);clf()
plot(tr,yr-SignalContinu(tr),'.');
xlabel('Temps (s)')
ylabel('Erreur (V)');
title('Erreur signal continu-signal reconstruit');
xgrid();
