clear 
stacksize('max');


//  Définition des fonctions
function ZoomPortAudio(win, x, y, idEvt)
// win est le numéro de la fenêtre
// x,y les coordonnées en pixel de la souris
// idEvt identificateur de l'événement
// Variables globales modifiables par la fonction ZoomPortAudio
// demi largeur du spectre affiché
global larX
// Fréquence minimale affichée
global minX
// Fréquence maximale affichée
global maxX
// Acquisition en cours
global acqEnCours
// Fenêtre graphique fermée ?
if idEvt==-1000 then return,end
// Bouton gauche souris pressé
if idEvt==0 then
    // passage des coordonnées pixels en système unité utilisateur
    [x,y]=xchange(x,y,'i2f')
    // le zoom est multiplié par 2 autour de la raie
    larX=larX/2;
    // calcul des minimums de la nouvelle fenêtre
    minX=max(x-larX,0);
    maxX=min(x+larX,Fe/2);
    // bouton droit souris pressé
else if idEvt==2 then
        // passage en zoom 1
        minX=0;
        maxX=Fe/2;
        larX=Fe/4;
// touche clavier souris pressée
else if idEvt>=32 then
            // l'acquisition est terminée
            acqEnCours=-10;
        end
    end
end
endfunction

// Variables globales du programme
global larX
global minX
global maxX
global acqEnCours

// Initialisation des graphiques
figure(0);
figure(1);
seteventhandler('')
// Fermeture des fenêtres graphiques
xdel(0);
xdel(1);

// Choix de la source du signal
sourceSignal=list('Source :',1,['Fichier WAV','Microphone via portaudio','Simulation']);
chxSource=x_choices('Origine du signal',list(sourceSignal));

select chxSource 
case 1 then
    // Demande du chemin et nom du fichier de données
    PathFileName=uigetfile('*.wav');
    // Lecture des données
    [y Fe]=wavread(PathFileName)
    // Si le son est sur deux voies on le transforme en une
    if  size(y,1)==2 then
        y =mean(y,'r');
    end
    N=length(y);
    // Variable temps en seconde
    t=(0:N-1)/Fe;
case 2 then
    // Saisie des paramètres de l'acquisition
    [ok,N,Fe]=getvalue("Paramètres de l''échantillonnage",["nombre d''échantillons";"Fe"],list("vec",1,"vec",1),["4096";"11025"]);
    t=(0:N-1)/Fe;
case 3  then
    // Saisie des paramètres de la simulation
    [ok,N,Fe]=getvalue("Paramètres de l''échantillonnage",["nombre d''échantillons";"Fe"],list("vec",1,"vec",1),["4096";"11025"]);
     // Paramètres des fréquences sinusoidales composant  le signal
    [ok,f1 A1 f2 A2 f3 A3]=getvalue("Signal avec trois fréquences",["Fréquence 1: f1(Hz) A1";" ";"Fréquence 2 f2(Hz) A2";" ";"Fréquence 3 f3(Hz) A3";" "],list("vec",1,"vec",1,"vec",1,"vec",1,"vec",1,"vec",1),["1000";"1";"1100";"1";"1300";"1"]);
     // Amplitude du bruit gaussien
    [ok v]=getvalue("Ajout de bruit gaussien",["Variance"],list("vec",1),["1"]);
    t=(0:N-1)/Fe;
    y=A1*sin(2*%pi*f1*t)+A2*sin(2*%pi*(f2)*t)+A3*sin(2*%pi*f3*t);
    w=v*rand(y,'n');
    y=y+w;
end

// Choix de la méthode d'analyse spectrale
listeMethode=list('Méthode :',1,['Périodogramme Welch','ARMA','Capon','Lagunas']);
chxMethode=x_choices('Estimation spectrale',list(listeMethode));

// Selon la méthode on a besoin de paramètres supplémentaires
select chxMethode 
case 1 then
    // Périodogramme de Welch
    // Largeur de fenêtre et pas de déplacement de la fenêtre
     [ok,M1 M2]=getvalue("Paramètres périodogramme Welch",["Largeur fenêtre";"Largeur recouvrement"],list("vec",1,"vec",1),["1024";"512"]);
    fenetre=['re';'tr';'hn';'hm';'kr';'ch'];
    // Choix de la fenêtre de pondération
    listeFenetre=list('Fenêtre',1,['Rectangle','Triangle','Hann','Hamming']);
    chxFen=x_choices('Type de fenêtres pour le périodogramme',list(listeFenetre));
case 2 then
    //Nombre de pôles et de zéros du modèle ARMA
    [ok,nbPole,nbZero,M]=getvalue("Paramètres ARMA",["Pôle";"Zéro";"Largeur fenêtre"],list("vec",1,"vec",1,"vec",1),["16";"1";"1024"]);
case 3 then
    //Nombre de pôles
    [ok,nbPole,M]=getvalue("Paramètres Capon",["Nombre";"Largeur fenêtre"],list("vec",1,"vec",1),["16";"1024"]);
case 4 then
    //Nombre de pôle 
    [ok,nbPole,M]=getvalue("Paramètres lagunas",["Nombre";"Largeur fenêtre"],list("vec",1,"vec",1),["16";"1024"]);
end

// Gestion de l'arrêt de la boucle while
acqEnCours=1; 
// Premier tour dans la boucle while
init=1;  
// Zoom 1 affichage entre 0 et Fe/2
minX=0;
maxX=Fe/2;
larX=Fe/4;

while acqEnCours>0
    acqEnCours=0;
    if chxSource==2 then
        y=pa_recordwav(N,Fe,1);
        //Ajout de 1 pour passer à un ou bien -9 si l'utilisateur souhaite arrêter
        acqEnCours = acqEnCours+1; 
    end
    // Analyse spectrale du signal avec la méthode choisie    
    select chxMethode 
    case 1 then
        [Y ]=pspect(M2,M1,fenetre(chxFen),y);
        f=(0:M1-1)/M1*Fe;
    case 2 then
        // Calcul du modèle ARMA
        [arc resid]=armax1(nbPole,nbZero,0,y,rand(1,size(y,2),'n'));
        mArmaSci=poly(arc(3),'z','coeff')/poly(arc(2),'z','coeff');
        // Analyse spectrale pour M points du modèle trouvé
        [Y,f]=frmag(mArmaSci,M);
        f=f*Fe;
    case 3 then
        // Calcul de matrice de corrélation
        a=correlmx(y',nbPole,'hermitian');
        // Analyse spectrale du signal pour M points
        [Y f]=parafrep(a,M,'capon')
        f=f'*Fe;
    case 4 then
        // Calcul de matrice de corrélation
        a=correlmx(y',nbPole,'hermitian');
        // Analyse spectrale du signal pour M points
        [Y f]=parafrep(a,M,'lagunas')
        f=f'*Fe;
    end
    ind=find(f>minX & f<=maxX & abs(Y)>0);
    // Graphique du signal temporel
    figure(1);clf();
    plot(t,y);
    // Analyse spectrale du signal
    figure(2);clf();
    // Mise en place du gestionnaire d'événements
    if init==1
        seteventhandler('ZoomPortAudio')
        init=0;
    end
    // Tracé du spectre en log    
    plot2d('nl',f(ind),abs(Y(ind)))
    xlabel('Frequence (Hz)');
    ylabel('Puissance (dB)');
    title(['Estimation spectrale :'+listeMethode(3)(chxMethode)]);
    xgrid();
end

