clear;
function  n=DetectionExt(v)
// recherche du maximum de v
    vMax=max(v);
// recherche des valeurs inférieures à 1/3 max et annulation de ces valeurs
    ind=find(v<vMax/3);
    v(ind)=0;
// calcul de la pente à droite de chaque élément de v
    d=sign(diff(v));
// Produit des pentes à droite et à gauche des éléments de v
    s=d(2:$).*d(1:$-1);
// Si le produit est négatif on a maximum local et on le marque à 1
    ind=find(s>0);
    s(ind)=0;
    ind=find(s<0);
    s(ind)=d(ind);
    n=[0  s  0];
endfunction
// Fréquence d'échantillonnage Fe et N nombre d'échantillons
Fe=11025;
N=2048;
// Caractéristiques initiales des fréquences 1 et 2 présentes dans le signal
A1=1;A2=1;
// La fréquence 1 est un multiple de Fe/N
indNu1=58;
nu1=indNu1*Fe/N;
// Choix de la fenêtre par l'utilisateur
fenetre=['re';'tr';'hn';'hm';'kr';'ch'];
// Construction d'une liste compatible avec la fonction x_choices
// Premier élément titre, indice du choix par défaut, les choix possibles
listeFenetre=list('Nom des fenêtres : ',1,['Rectangle','Triangle','Hann','Hamming','Kaiser','Chebyshev']);
chxFen=x_choices('Comparaison de deux fenêtres',list(listeFenetre));
// chxfen est l'indice du bouton cliqué par l'utilisateur
// Pour les fenêtres Chebyshev et Kaiser un paramètre supplémentaire est nécessaire
if  chxFen==5  then
    [ok beta]=getvalue('Paramètre de la fenêtre Kaiser','Valeur du paramètre',list('row',1),"10")
end
if  chxFen==6  then
    [ok beta]=getvalue('Paramètre de la fenêtre Chebyshev','Valeur du paramètre compris entre 0 et 0.5',list('row',1),"0.2")
end
// Construction de la fenêtre de pondération avec N éléments
select(chxFen)
case  5  then
    w=window(fenetre(chxFen),N,beta);
case  6  then
    w=window(fenetre(chxFen),N,[beta -1]);
else
    w=window(fenetre(chxFen),N);
end
// Ajout de 0 (63*N zéros) à la fenêtre de pondération pour augmenter la résolution spectrale
P=64;
w1=[zeros(1,P*N) ];
// La symétrie de la fenêtre est conservée
w1(1:N/2)=w(1:N/2);
w1(P*N:-1:P*N-N/2+1)=w(N:-1:N/2+1);
figure(1);
clf();
subplot(2,1,1);
plot((0:N-1),w)
xlabel('Echantillons');
ylabel('Gain')
xgrid();
subplot(2,1,2);
W=fft(w1);
indice=(0:N)
plot(indice/(N*P),(W(indice+1)/max(abs(W))))
xlabel('Fréquence normalisée')
ylabel('$\frac{X(k)}{X_{max}}$')
xgrid();
D=(N-1)/Fe;     // Largeur de la fenêtre temporelle
t=(0:N-1)/Fe;   // Valeur du temps en seconde
posRaie=[];     // Tableau des positions et amplitudes des raies
sepRaie=1;      // égale à 1 si les raies sont séparées après le fenêtrage
dRaie=-1;       // Distance minimale mesurée avec deux raies séparées
figure(2);
numGraph=1000;      // Variable d'indexation des graphiques
mkdir("animTmp");   // Dossier pour la sauvegarde des images
for  i=6:-0.05:0;
    nu2=nu1+i*Fe/N;
    drawlater()
     clf()
// signal composé de deux fréquences
// Le facteur sqrt(mean(w.^2)) permet de conserver l'énergie du signal
    x=[w/sqrt(mean(w.^2)).*(A1*sin(2*%pi*nu1*t)+A2*sin(2*%pi*nu2*t)  )  ]  ;
    subplot(2,1,1)
    plot(t,x)
    xlabel('Temps (s)');
    ylabel('Voltage (V)');
    xgrid();
// fft du signal
    X=abs(fft(x));
    f=(0:N-1)/N*Fe
    X=X*D/N;
// Fréquence attendue du pic2
    indNu2=fix(N/Fe*nu2);
// Marquage des pics à 1 entre les fréquences nu1-Fe/N et nu2+7Fe/N
    v=DetectionExt(X(indNu1-1:indNu2+7));      
    indPic=find(v==1);
    if  length(indPic)==2  then
// Les pics sont séparés
        posRaie=[posRaie;[nu1  nu2  indPic  X(indNu1-2+indPic(1))  X(indNu1+indPic(2)-2)]     ];
        if sepRaie==1
            dRaie=nu2-nu1;
            erreurMax=min(posRaie(:,6) ./ posRaie(:,5));
        end
    else          sepRaie=0;
// Les pics sont confondus
         posRaie=[posRaie;[nu1  nu2  -1  -2  -1  -1]     ];
    end                
    subplot(2,1,2)
// Tracé du spectre en fonction de la fréquence 
    plot(f,X)
    a=gca();
// On ne veut voir que la zone autour entre nu1 et nu2
    a.data_bounds=[nu1-5*Fe/N  0;  nu1+10*Fe/N  (A1+A2)*D/1.5];
    if length(indPic)>=1 then
// On trace un trait rouge d'épaisseur 3 horizontal au dessus du pic en nu1
        plot([nu1-10  nu1+10],[ X(indNu1-2+indPic(1))   X(indNu1-2+indPic(1))],'r','thickness',3)
    end
    if length(indPic)>=2 then
// On trace un trait horizontal au dessus du pic en nu2
        plot([nu2-10  nu2+10],[ X(indNu1+indPic(2)-2)   X(indNu1+indPic(2)-2)],'r','thickness',3)
    end
    xset("font",0,5);      
    if sepRaie==0
// Si les pics ne sont plus séparés on donne la résolution trouvée avec deux chiffres après la virgule
       xstring(nu1-10,A1*D/4,['Ratio (dB) ='+string(fix(-2000*log10(erreurMax))/100)])
       xstring(nu1-10,A1*D/2,['Resolution (Hz) ='+string(abs(fix(100*dRaie)/100))])
    end
    xset("font",0,3);
// On trace un trait rouge d'épaisseur 1 vertical en nu1 pour indiquer la valeur théorique
    plot([nu2  nu2],[A1*D/2  0],'r','thickness',1)
// On trace un trait bleu d'épaisseur 1 vertical en nu2 pour indiquer la valeur théorique
    plot([nu1  nu1],[A1*D/2  0],'b','thickness',1)
    xgrid();
    xlabel('Frequency (Hz)');
    ylabel('Voltage (V)');
    drawnow()
// Sauvegarde du graphique avec un nom dépendant du nombre tour effectué dans la boucle
    xs2jpg(2,["animTmp/"+fenetre(chxFen)+string(numGraph)+'.jpg'])
// Pause de 0.1s dans la boucle
    xpause(100000)
    numGraph=numGraph+1;
end
