clear;
stacksize('max')

// ***************************************************************
//  Définition des fonctions
// ***************************************************************
function F=TOC(f,psi,A,B)
// f expression littérale de la fonction
// psi expression littérale de psi
// A facteur d'échelle 
// B valeur de temps
// le résultat est une matrice comportant 
// autant de lignes que de facteur d'échelle A et autant de colonnes
// que de valeur de temps
    F=zeros(length(A),length(B));
    [nbl nbc]=size(F);
    // valeur du facteur d'échelle
    for i=1:nbl
        a=A(i);
    // Valeur de temps
        for j=1:nbc
            b=B(j);
            // Intégration du produit de l'ondelette par la fonction 
            // au point a,b
            w=inttrap(t,eval(f).*eval(psi));
            F(i,j)=w;
        end
    end
endfunction
// ***************************************************************
function s=OT_Mexicain(t,a,b)
    s=2/(%pi^0.25*sqrt(3)).*(1-((t-b)/(a)).^2).*exp(-1/2*((t-b)/(a)).^2)/sqrt(a);
endfunction
// ***************************************************************
function y=Porte(x)
    y=zeros(size(t,1),size(t,2));
    ind=find(x>-0.5 & x<0.50);
    if length(ind)>0
        y(ind)=1;
    end
endfunction
// ***************************************************************
function y=Triangle(t)
    y=zeros(size(t,1),size(t,2));
    ind=find(t>-1/2 & t<=0);
    y(ind)=1+2*t(ind);
    ind=find(t>0 & t<=1/2);
    y(ind)=1-2*t(ind);
endfunction
// ***************************************************************

// Échelle du signal
p=6
// Nombre d'échantillon
N=2^p;
// Durée du signal
D=1;
// Expression littérale de la fonction
fct='Porte((t-D/10)/(D/10))+Triangle((t-D/3)/(D/3))+Porte((t-3*D/4)/(D/2)).*sin(2*%pi/(D/4)*t)+0.2*Porte((t-7*D/8)/(D/4)).*sin(2*%pi/(D/11.14)*t)';
t=0:1/203:D;
// Valeurs de temps où la transformée en ondelettes sera calculée
b=t;
// Valeurs d'échelle où la transformée en ondelettesvaleur sera calculée
a=2.^(0:0.05:p)/N;
// Calcul de la transformée en ondelettes continue
w=TOC(fct,'OT_Mexicain(t,a,b)',a,b);


// Représentation graphique du résultat sous forme d'image
figure(1);clf();
// Choix d'une palette de 256 couleurs
nbCouleur=256;
// Palette dégradée de gris
p = graycolormap(nbCouleur);
// normalisation du résultat entre 0 et nbCouleur
// avec maximum en noir et minimum en blanc
ww=((w-max(w))/(min(w)-max(w)))*(nbCouleur-1);
// Tracé de l'image de la matrice
Matplot(fix(ww));
// Changement de la palette
f=gcf();f.color_map=p;f.background=nbCouleur;


figure(2);
clf();
// Tracé de la fonction dont on veut l'ondelette
subplot(2,1,1)
plot(t,eval(fct),'r')
xlabel('$ Temps(s) $')
ylabel('$ f(t)$')
xgrid();
// Tracé de transformée en ondelette au facteur d'échelle 0 et 6
subplot(2,1,2)
plot(b,w(1,:),'b-')
plot(b,w(length(a),:),'r-')
xlabel('$ Temps(s) $')
ylabel('$ W_{f} $')
xgrid();
// Calcul identique avec les fonctions de la toolbox SWT
nn=1:N;
// Période d'échantillonnage
Te=1/64;
te=(0:N-1)*Te;
t=te;
// Évaluation de la fonction
ye=eval(fct);
// Calcul de l'ondelette continue
c=cwt(ye,nn,'mexh');
// Représentation graphique sur la même fenêtre graphique
subplot(2,1,1)
plot(te,ye,'+')
xgrid();
subplot(2,1,2)
plot(te,c(1,:)*sqrt(Te),'b+')
plot(te,c( length(nn),:)*sqrt(Te),'ro')
xgrid();
legend('$ W_f{1,t} $','$ W_f{Te,t} $','$ W_f{1,t}(cwt) $','$ W_f{Te,t}(cwt) $');

// Représentation graphique du résultat sous forme d'image en utilisant cwtplot
figure(3);clf();
cwtplot(c,nn,%f,3);
f=gcf();
p = graycolormap(256);
f.color_map=p;f.background=256;
