clear;
// ***************************************************************
//  Définition des fonctions
// ***************************************************************
function y=Porte(x)
    y=zeros(size(t,1),size(t,2));
    ind=find(x>-0.5 & x<0.50);
    if length(ind)>0
        y(ind)=1;
    end
endfunction
// ***************************************************************
function y=Triangle(t)
    y=zeros(size(t,1),size(t,2));
    ind=find(t>-1/2 & t<=0);
    y(ind)=1+2*t(ind);
    ind=find(t>0 & t<=1/2);
    y(ind)=1-2*t(ind);
endfunction

// Échelle du signal
p=8
// Nombre d'échantillons
N=2^p;
// Durée du signal
D=1;
// Expression littérale de la fonction
fct='Porte((t-D/10)/(D/10))+Triangle((t-D/3)/(D/3))+Porte((t-3*D/4)/(D/2)).*sin(2*%pi/(D/4)*t)+0.4*Porte((t-7*D/8)/(D/4)).*sin(2*%pi/(D/11.14)*t)';
// Période d'échantillonnage
Te=1/N;
// Temps échantillonné
te=(0:N-1)*Te;
t=te;
// Évaluation de la fonction au point t
y=eval(fct);
// Ondelette de Daubechie avec 6 moments nuls
wavatome='db6';
// Valeurs de l'ondelette psi et de la fonction d'échelle phi au point xval
[phidb,psidb,xval]=wavefun(wavatome,1);
// Position des valeurs non nulles de psi 
ind=find(abs(psidb)>0);
// Suppression des valeurs nulles aux extrémités 
psidb=psidb(min(ind):max(ind));
nbPsi=size(psidb,2);
// Position des valeurs non nulles de phi 
ind=find(abs(phidb)>0);
// Suppression des valeurs nulles aux extrémités 
phidb=phidb(min(ind):max(ind));
// condition aux limites zero padding
dwtmode('zpd');
// calcul des coefficients d'ondelette discret sur 3 niveaux
[w pos]=wavedec(y,3,wavatome)
// Calcul des coefficients d'ondelette en utilisant la convolution
// Niveau 1 coefficient de détails 
wconv=conv([0*y y 0*y],psidb($:-1:1));
dy2=wconv(N+2:2:2*N+nbPsi-2)/sqrt(2);
// Niveau 1  coefficient d'approximation 
wconv=conv([0*y y 0*y],phidb($:-1:1));
y2=wconv(N+2:2:2*N+nbPsi-2)/sqrt(2);
// Niveau 2 coefficient de détails 
N2=size(y2,2);
wconv=conv([0*y2 y2 0*y2],psidb($:-1:1));
dy4=wconv(N2+2:2:2*N2+nbPsi)/sqrt(2);
// Niveau 2  coefficient d'approximation 
wconv=conv([0*y2 y2 0*y2],phidb($:-1:1));
y4=wconv(N2+2:2:2*N2+nbPsi)/sqrt(2);
// Niveau 3 coefficient de détails 
N4=size(y4,2);
wconv=conv([0*y4 y4 0*y4],psidb($:-1:1));
dy8=wconv(N4+2:2:2*N4+nbPsi)/sqrt(2);
// Niveau 3  coefficient d'approximation 
wconv=conv([0*y4 y4 0*y4],phidb($:-1:1));
y8=wconv(N4+2:2:2*N4+nbPsi)/sqrt(2);

// reconstruction du niveau 2 en utilisant les coefficients du niveau 3
yr4=(conv(dyadup(y8),phidb)+conv(dyadup(dy8),psidb))/sqrt(2);
yr4=yr4(size(psidb,2):pos(3)+size(psidb,2)-1);
// reconstruction du niveau 1 en utilisant les coefficients du niveau 2
yr2=(conv(dyadup(yr4),phidb)+conv(dyadup(dy4),psidb))/sqrt(2);
yr2=yr2(size(psidb,2):pos(4)+size(psidb,2)-1);
// reconstruction du niveau 0 en utilisant les coefficients du niveau 1
yr1=(conv(dyadup(yr2),phidb)+conv(dyadup(dy2),psidb))/sqrt(2);
yr1=yr1(size(psidb,2):pos(5)+size(psidb,2)-1);
// Tracé de la fonction avec sa réprésentation à différentes échelles (niveaux)
figure(0);clf();
// Niveau 0
subplot(4,1,1);
plot(yr1,'ro')
plot(y,'b+');
title('Fonction originale et reconstruite');
xgrid;
// Niveau 1
subplot(4,1,2)
plot(y2,'r-+')
title('Coefficient d''approximation niveau 1');
legend('conv','dwt')
xgrid;
// Niveau 2
subplot(4,1,3)
plot(y4,'r-+')
title('Coefficient d''approximation niveau 2');
xgrid;
// Niveau 3
subplot(4,1,4)
plot(y8,'r-+')
plot(w(1:pos(1)),'bo')
title('Coefficient d''approximation niveau 3');
legend('conv','dwt')
xgrid;
// Tracé des coefficients de détails
figure(1);clf();
// Niveau 1
subplot(4,1,2)
plot(dy2,'r-+')
plot(w(sum(pos(1:3))+1:sum(pos(1:3))+pos(4)),'bo')
title('Coefficients de détails niveau 1')
xgrid;
legend('conv','dwt')
// Niveau 2
subplot(4,1,3)
plot(dy4,'r-+')
plot(w(sum(pos(1:2))+1:sum(pos(1:2))+pos(3)),'bo')
title('Coefficients de détails niveau 2')
xgrid;
legend('conv','dwt')
// Niveau 3
subplot(4,1,4)
plot(dy8,'r-+')
plot(w(sum(pos(1))+1:sum(pos(1))+pos(2)),'bo')
title('Coefficients de détails niveau 3')
xgrid;
legend('conv','dwt')

