clear()
stacksize('max');

// ***************************************************************
//  Définition des fonctions
// ***************************************************************

// Transformée de Fourier à court terme du signal x
function X=TFCT(x,w,pas,signe)
    // w fenêtre de pondération
    // pas : avance de la fenêtre
    larFen=length(w);
    if signe==-1 then
        // Transformée directe
        N=length(x);
        // origine de fenêtres de pondération pour la discrétisation de tau 
        origFen=1:pas:N-1;
        // Résultat matrice d'un nombre de lignes égal à larFen (fréquences) 
        // et d'un nombre de colonnes égal au nombre de valeurs de tau
        X=zeros(larFen,length(origFen));
        for i=1:length(origFen)
            // extraction de la fenêtre
            indFen=origFen(i):min(origFen(i)+larFen-1,N);
            nbZeroAjoute=larFen-(min(origFen(i)+larFen-1,N)-origFen(i)+1);
            xf=[x(indFen);zeros(nbZeroAjoute,1)];
            // Transformée de Fourier locale pondérée par w
            X(:,i)=fft(xf.*w);
        end
    else
        // Transformée inverse
        // Dans le cas inverse signe est égal au nombre d'échantillons du signal
        X=zeros(signe,1);
        // Coefficient de normalisation de la transformée inverse
        sw2=zeros(signe,1);
        // x est la matrice des transformées locales
        [nbFreq nbSpectre ]=size(x);
        // Inversion de la transformée de fourier locale
        for i=1:nbSpectre 
            ind=(i-1)*pas+1:min((i-1)*pas+larFen,length(X));
            xx=ifft(x(:,i)).*w;
            X(ind)= X(ind)+xx(1:ind($)-ind(1)+1);
            sw2(ind)=sw2(ind)+w(1:ind($)-ind(1)+1).^2;
        end
        ind=find(sw2==0)
        if length(ind)>0
            disp('Attention fenêtre avec valeur nulle. Reconstruction incomplète');
        end
        ind=find(abs(sw2)>0);
        X(ind) = X(ind) ./ sw2(ind);
    end    
endfunction

// ***************************************************************

// Gestionnaire d'événements pour saisir un point de coordonnée Ox,Oy de la trajectoire 
function SelectPoint(win, x, y, ibut)
    // Les variables globales Ox, Oy et acqEnCours sont modifiables par la fonction
    global Ox
    global Oy
    global acqEnCours
    // Fenêtre graphique fermée?
    if ibut==-1000 then return,end
    // Bouton gauche pressé?
    if ibut==10 then
         [Ox,Oy]=xchange(x,y,'i2f')
    end
    // Touche clavier  pressé?
    if ibut==12 then
    // L'acquisition est terminée
        acqEnCours=-10;
    end
endfunction

// ***************************************************************

// Création d'une trajectoire à partir de points
function m=SelectTrajectoire(c)
    // Variables globales modifiables par la fonction
    // (Ox,Oy) coordonnées d'un point de la trajectoire  
    global Ox
    global Oy
    global acqEnCours
    m=[];
    Ox=-1;
    // la boucle est activée
    acqEnCours=1;
    init=1;
    while acqEnCours>0
        if init==1
            seteventhandler('SelectPoint')
            init=0;
        end
        // Est-ce qu'un nouveau point a été cliqué?
        if Ox~=-1  then
            // oui on ajoute le point(Ox,Oy)
            m=[m [Ox;Oy]];
            // Tracé de la nouvelle trajectoire
            plot(m(1,:),m(2,:),c+'o')
            plot(m(1,:),m(2,:),c+'-')
            // Attente du nouveau point
            Ox=-1;
        end
        // Boucle arrêtée et aucun point saisi
        if acqEnCours<0  & length(m)==0 then
            // Réactivation de la boucle
            acqEnCours=1;
            disp('Il faut saisir au moins un point');
        end
    end
    // Désactivation du gestionnaire d'événements
    seteventhandler('')
endfunction

// ***************************************************************

function M=TrajectoireLineaire(P,t)
// Interpolation linéaire entre deux points à chaque pas de temps 
    M=[];
    // Nombre de pas de temps
    N=length(t);
    // Y a-t-il deux points ?
    if size(P,2)>1 then 
        // Nombre de segments 
        nb=size(P,2)-1;
        deb=1;
        // Nombre de pas de temps entre deux points
        pas=fix(N/(size(P,2)-1));
        fin=deb+pas-1;
        for i=1:nb
            // Vitesse entre deux points
            v=[P(:,i+1)-P(:,i)]/(t(fin)-t(deb));
            // Vecteur déplacement 
            T=v*(t(deb:fin)-t(deb));
            // Coordonnées des nouveaux points
            T(1,:)=T(1,:)+P(1,i);
            T(2,:)=T(2,:)+P(2,i);
            // Ajout des points à la trajectoire
            M=[M T];
            deb=fin+1;
            fin=deb+pas-1;
            if i==nb-1 then
                fin=N;
            end
        end   
    else 
        // Lorsqu'il n'y a un seul point on le duplique N fois
        M=repmat(P,1,N);
    end
endfunction

// ***************************************************************


// Utilisation des descripteurs de Fourier pour interpoler la trajectoire
function z=FourierDescripteur(p,N)
    pp=real(p(1,:))+%i*(p(2,:));
    c=fft(pp);
    Nc=length(c);
    Z=zeros(1,N);
    if modulo(Nc,2)==0 then
        b=Nc/2;
    else
        b=(Nc+1)/2;
    end
    for i=1:b
        Z(i)=N/Nc*c(i);
    end
    for i=b+1:length(c)
        Z(N-length(c)+i)=N/Nc*c(i);
    end
    z=ifft(Z);
    z=[real(z);imag(z)];
endfunction


//*****************************************************************
// Programme principal
//*****************************************************************

// Variables globales
global Ox
global Oy
global acqEnCours
// Les trajectoires sont saisies sur la figure 0
// dans un carré de  1000m x 1000m
figure(0);
clf;
plot([0 1000],[0 1000],'w')
xgrid(10);
title('Trajectoire de l''observateur ');
// Saisie de la trajectoire de l'observateur en rouge
c='r';
O=SelectTrajectoire(c)
xdel(0);
figure(0);
clf;
// Tracé de la trajectoire de l'observateur en rouge 
// avec aux points de controles dex 'x'
plot([0 1000],[0 1000],'w')
plot([O(1,:)],[O(2,:)],c+'x') 
plot([O(1,:) O(1,1)],[O(2,:) O(2,1)],c+'-') 
xgrid(10);
legend(' ','Observateur')
// Saisie de la trajectoire de la source en bleu
c='b';
S=SelectTrajectoire(c)
plot([S(1,:) S(1,1)],[S(2,:) S(2,1)],c+'-') 
legend(' ',' ','Observateur','Source',' ')
// Nombre d'échantillons et fréquence d'acquisition du son
[ok,Nb,Fe]=getvalue("Paramètre de l''échantillonnage",["Nombre d''échantillons";"Fe"],list("vec",1,"vec",1),["2^17";"11025"]);
// Quel type d'interpolation de trajectoire veut l'utilisateur?
sourceSignal=list('Trajectoire ',1,['Linéaire','Descripteur de Fourier']);
depLineaire=x_choices('Type de trajectoire',list(sourceSignal));
t=(0:Nb-1)/Fe;
if depLineaire==1 then
// Interpolation linéaire
    Obs=TrajectoireLineaire(O,t);
    Src=TrajectoireLineaire(S,t);
else
// Interpolation avec les descripteurs de Fourier
    Src=FourierDescripteur(S,Nb);
    Obs=FourierDescripteur(O,Nb);
    plot(Src(1,:),Src(2,:),'b')
    plot(Obs(1,:),Obs(2,:),'r')
end
// Vitesse du son
c=340;
// Fréquences présentes dans le son émis par la source
f=[220 2*220 3*220];
// Amplitude des fréquences
Af=[0.4 0.2 0.30];
// Vecteur source observateur
OS=Src-Obs;
// Distance source observateur
OS=sqrt(OS(1,:).^2+OS(2,:).^2);
y=[];  
for j=1:length(f)
  // Equation d'onde à la position de l'observateur
   y=y+Af(j)*(sin(2*%pi*f(j)*(t-OS/c))./ OS.^2 )';
end
// fréquences perçues à chaque instant
fs=[f' f'*(1+diff(OS)/c*Fe)];
figure(1);
clf();
// Tracé du signal temporel en haut
subplot(2,1,1)
plot(t,y)
xlabel('temps (s)')
// Tracé des  fréquences perçues
subplot(2,1,2)
plot(t,fs')
xlabel('temps (s)')
ylabel('Frequence (Hz)')
// Analyse temps-fréquence
// Transformée de Fourier à court terme
// Largeur de la fenêtre pondération 2048
larFenetre=2048;
// Espacement des valeurs de tau élgal à 1/4 de 
// la largeur de la fenêtre de pondération
pasFenetre=larFenetre/4;
// Fenêtre de pondération Hamming
w=window('hm',larFenetre)';
// Transformée de Fourier à court terme
stft=TFCT(y,w,pasFenetre,-1);

figure(2);
// représentation graphique de la TFCT sous forme d'image
clf();
// Nombre de fréquences Nombre de spectres dans la TFCT
[nbFreq nbSpec]=size(stft);
// Image module de la TFCT normalisé entre 0 et 255
Matplot(abs(stft(nbFreq/2:-1:1,:))/max(abs(stft))*255);
// palette de 256 couleurs 
f=gcf();p = jetcolormap(256);f.color_map=p;f.background=-2;
// Construction des graduations et des légendes sur les axes
// Axe des abscisses 
// Légendes pour chaque graduation
legGradx=[];
// 5 graduations
nbGradX=5
// Position des graduations
px=[];
for i=0:nbGradX-1
         ind=fix(i*size(stft,2)/(nbGradX-1))+1;
         px=[px ind];
         legGradx=[legGradx; string(fix(ind*pasFenetre/Fe*100)/100)];
end
// Axe des ordonnées
// Légendes pour chaque graduation
legGrady=[];
// 10 graduations
nbGradY=10;
// Position des graduations
py=[];
for i=0:nbGradY-1
         ind=fix(i*nbFreq/2/(nbGradY-1))+1;
         py=[py ind];
         legGrady=[ string(fix((nbGradY-1-i)/(nbGradY-1)*Fe/2));legGrady];
end
// Structure des axes
aa=gca();
// Modification des légendes sur x
aa.x_ticks = tlist(["ticks", "locations", "labels"], px', legGradx);  
// Modification des légendes sur y
aa.y_ticks = tlist(["ticks", "locations", "labels"], py', legGrady);  
// L'axe est affiché en bas à gauche
aa.x_location = 'bottom';
aa.y_location = 'left';
xlabel('$ Temps(s) $')
ylabel('$ \nu (Hz) $'),
title('Transformée de Fourier à court terme');
// transformée de Wigner-Ville du signal analytique de y
wv=wigner(hilbert(y'),(w'),pasFenetre,larFenetre);
// Nombre de fréquences Nombre de spectres dans la transformée de WV
[nbSpecW nbFreqW ]=size(wv);
// représentation graphique de la transformée de WV sous forme d'image
figure(3);
clf();
// Image du module de la TF WV normalisé entre 0 et 255
Matplot(sqrt(abs(wv(:,nbFreqW:-1:1))'/max(abs(wv)))*255);
// palette de 256 couleurs 
f=gcf();p = jetcolormap(256);f.color_map=p;f.background=-2; 
// Les graduations sur l'axe des abscisses sont identiques à la TFCT
// Axe des ordonnées
// Légendes pour chaque graduation
legGrady=[];
// 10 graduations
nbGradY=10;
// Position des graduations
py=[];
for i=0:nbGradY-1
         ind=fix(i*nbFreqW/(nbGradY-1))+1;
         py=[py ind];
         legGrady=[ string(fix((nbGradY-1-i)/(nbGradY-1)*Fe/2));legGrady];
end
// Structure des axes
aa=gca();
// Modification des légendes sur x
aa.x_ticks = tlist(["ticks", "locations", "labels"], px', legGradx);  
// Modification des légendes sur y
aa.y_ticks = tlist(["ticks", "locations", "labels"], py', legGrady);  
// Les axes sont affichés en bas à gauche
aa.x_location = 'bottom';
aa.y_location = 'left';
xlabel('$ Temps(s) $')
ylabel('$ \nu (Hz) $')
title('Transformée de Wigner du signal analytique');
// transformée de Choi-Williams du signal analytique de y
[cw,T,F]=tfrcw(hilbert(y),1:pasFenetre:Nb,larFenetre);
figure(4);clf;
Matplot(sqrt(abs(cw(larFenetre:-1:1,:))/max(abs(cw)))*256);
f=gcf();p  =  jetcolormap(256);f.color_map=p;f.background=-2;
aa=gca();
aa.x_ticks = tlist(["ticks", "locations", "labels"], px', legGradx);  
aa.y_ticks = tlist(["ticks", "locations", "labels"], py', legGrady);  
aa.x_location = 'bottom';
aa.y_location = 'left';
xlabel('$ Temps(s) $')
ylabel('$ \nu (Hz) $')
title('Transformée de Choi-Williams du signal analytique');
 
